# SPDX-License-Identifier: CC0-1.0

{
  edition = 201909;

  description = "Genode packages";

  inputs.nixpkgs.url = "github:ehmry/nixpkgs?ref=genode";

  outputs = { self, nixpkgs }:
    let
      localSystems = [ "x86_64-linux" ];
      crossSystems = [ "aarch64-genode" "x86_64-genode" ];

      forAllLocalSystems = f:
        nixpkgs.lib.genAttrs localSystems (system: f system);

      forAllCrossSystems = f:
        with builtins;
        let
          f' = localSystem: crossSystem:
            let system = localSystem + "-" + crossSystem;
            in {
              name = system;
              value = f { inherit system localSystem crossSystem; };
            };
          list = nixpkgs.lib.lists.crossLists f' [ localSystems crossSystems ];
          attrSet = listToAttrs list;
        in attrSet;

      forAllSystems = f:
        (forAllCrossSystems f) // (forAllLocalSystems (system:
          f {
            inherit system;
            localSystem = system;
            crossSystem = system;
          }));

      nixpkgsFor = forAllSystems ({ system, localSystem, crossSystem }:
        if localSystem == crossSystem then
          import nixpkgs {
            inherit system;
            overlays = [ self.overlay ];
          }
        else
          import nixpkgs {
            inherit localSystem;
            crossSystem = {
              system = crossSystem;
              useLLVM = true;
            };
            config.allowUnsupportedSystem = true;
            overlays = [ self.overlay ];
          });

    in rec {

      overlay = import ./overlay;

      lib = forAllCrossSystems ({ system, localSystem, crossSystem }:
        nixpkgs.lib // (import ./lib {
          inherit system localSystem crossSystem;
          apps = self.apps.${system};
          genodepkgs = self;
          nixpkgs = nixpkgsFor.${system};
        }));

      legacyPackages = forAllSystems
        ({ system, localSystem, crossSystem }: nixpkgsFor.${system});
      # pass thru Nixpkgs

      packages = forAllCrossSystems ({ system, localSystem, crossSystem }:
        nixpkgs.lib.filterAttrs (n: v: v != null)
        nixpkgsFor.${system}.genodePackages);

      devShell = forAllLocalSystems (system:
        let
          pkgs = nixpkgsFor.${system};
          fhs = pkgs.buildFHSUserEnv {
            name = "genode-env";
            targetPkgs = pkgs:
              (with pkgs; [
                binutils
                bison
                expect
                flex
                git
                glibc.dev
                gnumake
                libxml2
                qemu
                tcl
                which
                xorriso
              ]);
            runScript = "bash";
            extraBuildCommands = let
              toolchain = pkgs.fetchzip {
                url =
                  "file://${packages.x86_64-linux-x86_64-genode.genodeSources.toolchain.src}";
                hash = "sha256-26rPvLUPEJm40zLSqTquwuFTJ1idTB0T4VXgaHRN+4o=";
              };
            in "ln -s ${toolchain}/local usr/local";
          };
        in pkgs.stdenv.mkDerivation {
          name = "genode-fhs-shell";
          nativeBuildInputs = [ fhs ];
          shellHook = "exec genode-env";
        });

      apps = let
        apps' = forAllCrossSystems ({ system, localSystem, crossSystem }:
          import ./apps {
            system = crossSystem;
            self = self.apps.${system};
            nixpkgs = nixpkgsFor.${system};
            nixpkgsLocal = nixpkgsFor.${localSystem};
            packages = self.packages.${system};
          });
      in apps' // { x86_64-linux = apps'.x86_64-linux-x86_64-genode; };

      checks = let
        checks' = forAllCrossSystems ({ system, localSystem, crossSystem }:
          import ./tests {
            inherit self;
            apps = self.apps.${system};
            localPackages = nixpkgsFor.${localSystem};
            genodepkgs = self.packages.${system};
            lib = self.lib.${system};
            nixpkgs = nixpkgsFor.${system};
            legacyPackages = self.legacyPackages.${system};
          });
      in checks' // {
        x86_64-linux = with builtins;
          foldl' (x: y: x // y) { } (attrValues checks');
      };
    };
}
